from rest_framework import routers
from .views import SoftwareViewSet, LicenceViewSet

router = routers.DefaultRouter()
router.register('software', SoftwareViewSet)
# router.register('licences', LicenceViewSet)


urlpatterns = router.urls



from extras.plugins import PluginConfig

class LicencesConfig(PluginConfig):
    name = "netbox_licences"
    verbose_name = "Netbox Licences"
    description = "Extension providing an licences listings functionality"

    version = "0.1"

    # Plugin author
    author = 'Tomasz Durda'
    author_email = 'tomekdur@wp.pl'

    # Configuration parameters that MUST be defined by the user (if any)
    required_settings = []

    # Default configuration parameter values, if not set by the user
    default_settings = {
    }

    # Base URL path. If not set, the plugin name will be used.
    base_url = 'licences'

    # Caching config
    caching_config = {}


config = LicencesConfig




from django.contrib import admin
from .models import SoftwareProvider, SoftwareType, Software, LicenceType, Licence

@admin.register(SoftwareProvider)
class SoftwareProviderAdmin(admin.ModelAdmin):
    list_display = ('name',)

@admin.register(SoftwareType)
class SoftwareTypeAdmin(admin.ModelAdmin):
    list_display = ('name',)

@admin.register(Software)
class SoftwareAdmin(admin.ModelAdmin):
    list_display = ('name','softtype','provider',)

@admin.register(LicenceType)
class LicenceTypeAdmin(admin.ModelAdmin):
    list_display = ('name',)

@admin.register(Licence)
class LicenceAdmin(admin.ModelAdmin):
    list_display = ('inventory_number',
        'licencetype',
        'date_created',
        'date_valid',
        'amount',
        'software',
        'tenant',
        'site'
    )



from rest_framework.viewsets import ModelViewSet
from netbox_licences.models import Licence, Software
from .serializers import SoftwareSerializer, LicenceSerializer

class SoftwareViewSet(ModelViewSet):
    queryset = Software.objects.all()
    serializer_class = Software

class LicenceViewSet(ModelViewSet):
    queryset = Licence.objects.all()
    serializer_class = Licence




from extras.plugins import PluginMenuButton, PluginMenuItem
from utilities.choices import ButtonColorChoices

menu_items = (
    PluginMenuItem(
        permissions = ['netbox_licences.view_softwareprovider'],
        link='plugins:netbox_licences:software_providers_list',
        link_text='Software Providers',
        buttons=(
            PluginMenuButton('home', 'Button A', 'fa fa-info', ButtonColorChoices.BLUE),
            PluginMenuButton('home', 'Button B', 'fa fa-warning', ButtonColorChoices.GREEN),
        )
    ),
    PluginMenuItem(
        permissions = ['netbox_licences.view_licence'],
        link='plugins:netbox_licences:licences_list',
        link_text='Licences',
    ),
)



from django import forms
from django_rq import get_queue

from .models import SoftwareProvider, Licence, Software

from mptt.forms import TreeNodeChoiceField

from utilities.forms import (
    APISelectMultiple,
    DynamicModelMultipleChoiceField,
    StaticSelect2Multiple,
    BootstrapMixin
)

from dcim.models import Site


class CommonLicencesFilterForm(BootstrapMixin, forms.ModelForm):

    q = forms.CharField(required=False, label="Search")

    class Meta:
        model = SoftwareProvider
        fields = [
            "q",
        ]

class LicencesFilterForm(BootstrapMixin, forms.ModelForm):
    q = forms.CharField(required=False, label="Search")

    inventory_number = forms.CharField(required=False, label="Inventory Number")

    # software = DynamicModelMultipleChoiceField(
    #     queryset=Software.objects.all(),
    #     required=False,
    #     to_field_name="software__slug",
    #     widget=APISelectMultiple(
    #         value_field="software__slug",
    #     )
    # )

    site = DynamicModelMultipleChoiceField(
        queryset=Site.objects.all(),
        required=False,
        to_field_name="site__slug",
         widget=APISelectMultiple(
            value_field="site__slug",
        )
    )

    class Meta:
        model = Licence
        fields = [
            "q",
            "inventory_number",
            "site",
            # "software"
        ]



import django_tables2 as tables
from utilities.tables import BaseTable, ToggleColumn

from .models import SoftwareProvider, Licence

class SoftwareProviderTable(BaseTable):
    class Meta(BaseTable.Meta):
        model = SoftwareProvider
        fields = (
            "name",
            "full_name"
        )

class LicenceTable(BaseTable):
    class Meta(BaseTable.Meta):
        model = Licence
        fields = (
            "inventory_number",
            "licencetype",
            "date_created",
            "date_valid",
            "amount",
            "software",
            "tenant",
            "site"
        )



from django.db import models

class NamedModel(models.Model):
    """ Base for the most models """
    name = models.CharField(max_length=50)

    class Meta:
        ordering = [ 'name' ]
        abstract = True

    def __str__(self):
        return self.name

class SoftwareProvider(NamedModel):
    full_name = models.CharField(max_length=64)


class SoftwareType(NamedModel): pass

class Software(NamedModel):
    provider = models.ForeignKey(on_delete=models.deletion.CASCADE,to='SoftwareProvider')
    softtype = models.ForeignKey(on_delete=models.deletion.CASCADE,to='SoftwareType')

class LicenceType(NamedModel): pass

class Licence(models.Model):
    inventory_number =  models.CharField(unique=True, max_length=50)
    licencetype =  models.ForeignKey(null=True,on_delete=models.deletion.CASCADE,to='LicenceType')
    date_created =  models.DateTimeField(auto_now=True)
    date_valid =  models.DateField()
    amount = models.IntegerField()
    software = models.ForeignKey(blank=True, null=True,on_delete=models.deletion.CASCADE,to='Software')
    tenant =  models.ForeignKey(blank=True, null=True, on_delete=models.deletion.CASCADE,to='tenancy.Tenant')
    site =  models.ForeignKey(blank=True, null=True,on_delete=models.deletion.CASCADE,to='dcim.Site')




from rest_framework.serializers import ModelSerializer
from netbox_licences.models import Licence, Software
import sys

sys.exit()

class SoftwareSerializer(ModelSerializer):
    class Meta:
        model = Software
        fields = ('id', 'name', 'provider', 'softtype')


class LicenceSerializer(ModelSerializer):
    class Meta:
        model = Licence
        fields = ('id', 'inventory_number', 'licencetype', 'date_created', 'date_valid', 'amount', 'software', 'tenant')



from django.urls import path
from . import views


urlpatterns = [
    path('software-providers/', views.SoftwareProviderListView.as_view(), name='software_providers_list'),
    path('licences/', views.LicenceListView.as_view(), name='licences_list'),
]



from django.db import migrations,models

class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('tenancy', '0003_unicode_literals')
    ]

    operations = [
        migrations.CreateModel(
            name='SoftwareProvider',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False)),
                ('full_name', models.CharField(max_length=100)),
                ('name', models.CharField(max_length=50)),
            ],
            options = {
                'ordering': ['name']
            }
        ),
        migrations.CreateModel(
            name='SoftwareType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=50)),
            ],
            options = {
                'ordering': ['name']
            }
        ),
        migrations.CreateModel(
            name='Software',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=50)),
                ('provider', models.ForeignKey(on_delete=models.deletion.CASCADE,to='SoftwareProvider')),
                ('softtype', models.ForeignKey(on_delete=models.deletion.CASCADE,to='SoftwareType')),
            ],
            options = {
                 'ordering': ['name']
            },
        ),
        migrations.CreateModel(
            name='LicenceType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=50)),
            ],
            options = {
                'ordering': ['name']
            }
        ),
        migrations.CreateModel(
            name='Licence',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False)),
                ('inventory_number', models.CharField(unique=True, max_length=50)),
                ('licencetype', models.ForeignKey(null=True,on_delete=models.deletion.CASCADE,to='LicenceType')),
                ('date_created', models.DateTimeField(auto_now=True)),
                ('date_valid', models.DateField()),
                ('amount', models.IntegerField()),
                ('software', models.ForeignKey(blank=True, null=True,on_delete=models.deletion.CASCADE,to='Software')),
                ('tenant', models.ForeignKey(blank=True, null=True,on_delete=models.deletion.CASCADE,to='tenancy.Tenant')),
                ('site', models.ForeignKey(blank=True, null=True,on_delete=models.deletion.CASCADE,to='dcim.Site')),
            ],
            options = {
                'ordering': ['name']
            }
        ),
        # migrations.CreateModel(
        #     name='SoftwareSupport',
        #     fields=[
        #         ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False)),
        #         ('')
        #     ]
        # ),
        # migrations.CreateModel(
        #     name='HardwareSupport',
        #     fields=[
        #         ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False)),
        #         ('')
        #     ]
        # ),
    ]



from django.shortcuts import render
from django.http import HttpResponse
from django.views.generic import View

from django.contrib.auth.mixins import PermissionRequiredMixin
from utilities.views import BulkDeleteView, BulkImportView, ObjectEditView, ObjectListView

from .filters import (
    CommonLicencesFilter,
    LicencesFilter
)
from .forms import (
    CommonLicencesFilterForm,
    LicencesFilterForm
)
from .models import (
    SoftwareProvider,
    Licence
)
from .tables import (
    SoftwareProviderTable,
    LicenceTable
)

class SoftwareProviderListView(PermissionRequiredMixin, ObjectListView):
    permission_required = 'netbox_licences.view_softwareprovider'
    queryset = SoftwareProvider.objects.all()
    filterset = CommonLicencesFilter
    filterset_form = CommonLicencesFilterForm
    table = SoftwareProviderTable
    template_name = "netbox_licences/software_providers_list.html"


class LicenceListView(PermissionRequiredMixin, ObjectListView):
    permission_required = 'netbox_licences.view_licence'
    queryset = Licence.objects.all()
    filterset = LicencesFilter
    filterset_form = LicencesFilterForm
    table = LicenceTable
    template_name = "netbox_licences/licences_list.html"



import django_filters
from django.db.models import Q

from dcim.models import Site

from utilities.filters import (
   BaseFilterSet,
   MultiValueCharFilter,
   MultiValueMACAddressFilter,
   MultiValueNumberFilter,
   NameSlugSearchFilterSet,
   TagFilter,
   TreeNodeMultipleChoiceFilter,
)

from extras.filters import CustomFieldFilterSet, LocalConfigContextFilterSet, CreatedUpdatedFilterSet

from .models import SoftwareProvider, Software, Licence


class CommonLicencesFilter(BaseFilterSet):
    q = django_filters.CharFilter(method="search", label="Search")

    class Meta:
        model = SoftwareProvider
        fields = ["id", "name"]

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = (
            Q(id__icontains=value)
            | Q(name__icontains=value)
        )
        return queryset.filter(qs_filter)

class LicencesFilter(django_filters.FilterSet):

    q = django_filters.CharFilter(method="search", label="Search")

    inventory_number = django_filters.CharFilter(method="search_inv_num", label="Inventory Name (only)")

    software = TreeNodeMultipleChoiceFilter(
        queryset = Software.objects.all(),
        field_name = "software",
        lookup_expr='in',
        to_field_name='software__slug',
        label = "Software"
    )

    site = TreeNodeMultipleChoiceFilter(
        queryset = Site.objects.all(),
        field_name = "site",
        lookup_expr='in',
        to_field_name='site__slug',
        label = "Site"
    )

    class Meta:
        most = Licence
        fields = [
            "id",
            "inventory_number",
            "date_created",
            "date_valid",
            "amount",
            "software",
            "site",
            "tenant"
        ]

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = (
            Q(id__icontains=value)
            | Q(inventory_number__icontains=value)
            | Q(software_number__icontains=value)
            | Q(site__icontains=value)
            | Q(tenant__icontains=value)
        )
        return queryset.filter(qs_filter)

    def search_in_field(self, search, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = (
            search
        )
        return queryset.filter(qs_filter)

    def search_inv_num(self, queryset, name, value):
        return self.search_in_field(Q(inventory_number__icontains=value), queryset, name, value)

    def search_software(self, queryset, name, value):
        return self.search_in_field(Q(software__icontains=value), queryset, name, value)
